package widebase.stream.socket.rq

import java.io.IOException
import java.util.concurrent.TimeUnit

import org.jboss.netty.channel.SimpleChannelUpstreamHandler

import scala.collection.mutable.LinkedHashMap

import widebase.db.table.Table

import widebase.stream.codec. {

  DoneMessage,
  ForbiddenMessage,
  LoginRequiredMessage,
  UnauthorizedMessage

}

import widebase.stream.codec.rq. {

  BrokerDecoder,
  ConsumerEncoder,
  EndOfDayMessage,
  SubscribeMessage,
  TableMessage,
  UnsubscribeMessage

}

import widebase.stream.handler.rq. { ConsumerHandler, RecordListener }
import widebase.stream.socket.ClientLike

/** A consumer.
 *
 * @param listener of consumer
 * @param host of broker
 * @param port of broker
 *
 * @author myst3r10n
 */
class Consumer(
  val listener: RecordListener,
  val host: String,
  val port: Int)
  extends ClientLike {

  override protected def decoder = new BrokerDecoder
  override protected def encoder = new ConsumerEncoder
  override protected def handlers =
    LinkedHashMap("consumer" -> new ConsumerHandler(listener))

  override def login(
    username: String,
    password: String,
    timeout: Int = 60): Consumer = {

    val future = connect(host, port).awaitUninterruptibly

    if(!future.isSuccess)
      throw new IOException(future.getCause)

    try {

      super.login(username, password, timeout).asInstanceOf[Consumer]

    } catch {

      case e =>
        val future = disconnect.awaitUninterruptibly

        if(!future.isSuccess)
          throw new IOException(future.getCause)

        throw e

    }
  }

  /** Subscribe table.
   *
   * @param name of table
   * @param selector of table
   * @param timeout of response
   *
   * @return itself
   */
  def subscribe(
    name: String,
    selector: String = "",
    timeout: Int = 60) = {

    val future =
      channel.write(new SubscribeMessage(name, selector)).awaitUninterruptibly

    if(!future.isSuccess)
      throw new IOException(future.getCause)

    val message = reader.read(timeout, TimeUnit.SECONDS)

    if(message == null)
      throw new IOException(new NullPointerException)

    message match {

      case message: DoneMessage =>
      case message: ForbiddenMessage => throw new ForbiddenMessage
      case message: LoginRequiredMessage => throw new LoginRequiredMessage
      case message: UnauthorizedMessage => throw new UnauthorizedMessage

    }

    this

  }

  /** Subscribe table.
   *
   * @param timeout of response
   *
   * @return itself
   */
  def unsubscribe(timeout: Int = 60) = {

    val future = channel.write(new UnsubscribeMessage).awaitUninterruptibly

    if(!future.isSuccess)
      throw new IOException(future.getCause)

    this

  }
}

